import boto3
import logging

from typing import List, Tuple, Optional
from functools import reduce
from kong_config_builder.password import PasswordManager
from kong_config_builder.settings import KCB_AWS_REGION_NAME


class ParameterStoreAPI:
    def __init__(
        self,
        client=boto3.client("ssm", region_name=KCB_AWS_REGION_NAME),
        password=PasswordManager
    ):
        self._logger = logging.getLogger(__name__)
        self._client = client
        self._password_manager = password

        self._available_parameters_cache = {}

    def get(self, name: str) -> Optional[str]:
        value = self._available_parameters_cache.get(name, None)
        return value

    def put(
        self,
        name: str,
        value: str,
        tags: List[Tuple] = [],
        encrypt: bool = True,
        overwrite: bool = False
    ) -> None:
        _type = "SecureString" if encrypt else "String"
        _tags = list(map(lambda tag: {"Key": tag[0], "Value": tag[1]}, tags))
        try:
            self._client.put_parameter(
                Name=name,
                Value=value,
                Tags=_tags,
                Type=_type,
                Overwrite=overwrite
            )
            self._available_parameters_cache[name] = value
        except Exception as err:
            self._logger.error(err)

    def populate(self, namespace: str) -> None:
        parameters_list = self._get_parameters_by(namespace)

        for parameter in parameters_list:
            values = self._get_value_by(parameter)
            self._available_parameters_cache[parameter] = values[parameter]

    def _get_parameters_by(self, namespace: str) -> List:
        paginate = self._client.get_paginator("describe_parameters")
        parameter_filters = [{
            "Key": "Name",
            "Values": [namespace],
            "Option": "BeginsWith"
        }]
        iterator = paginate.paginate(ParameterFilters=parameter_filters)
        params = set()
        for page in iterator:
            for parameter in page["Parameters"]:
                params.add(parameter["Name"])

        return list(params)

    def _get_value_by(self, key: str):
        def objecter(current, next):
            current[next["Name"]] = next["Value"]
            return current
        parameters = self._client.get_parameters(
            Names=[key],
            WithDecryption=True
        )

        return reduce(objecter, parameters["Parameters"], {})



from typing import List
from dataclasses import dataclass
from kong_config_builder.base import BaseYamlObject


@dataclass
class Route(BaseYamlObject):
    name: str
    paths: List[str]
    strip_path: bool = False


@dataclass
class Service(BaseYamlObject):
    name: str
    host: str
    routes: List[Route]



from kong_config_builder.base import BaseYamlObject


def remove_none(obj):
    if isinstance(obj, BaseYamlObject):
        return remove_none(obj.__dict__)
    elif isinstance(obj, (list, tuple, set)):
        return list(remove_none(x) for x in obj if x is not None)
    elif isinstance(obj, dict):
        return dict((remove_none(k), remove_none(v))
                    for k, v in obj.items()
                    if k is not None and v is not None)
    else:
        return obj


def no_empty(dumper, data):
    values = remove_none(data)
    data.__dict__ = values
    return dumper.represent_yaml_object(
        BaseYamlObject.yaml_tag, data, BaseYamlObject,
        flow_style=BaseYamlObject.yaml_flow_style
    )


def noop(self, *args, **kw):
    pass



from typing import List
from dataclasses import dataclass
from kong_config_builder.base import BaseYamlObject


@dataclass
class HealthcheckUnhealthy(BaseYamlObject):
    interval: int
    timeouts: int
    http_failures: int


@dataclass
class HealthcheckHealthy(BaseYamlObject):
    interval: int
    successes: int
    http_statuses: List[int]


@dataclass
class HealthcheckActive(BaseYamlObject):
    type: str
    http_path: str
    timeout: float
    concurrency: int
    healthy: HealthcheckHealthy
    unhealthy: HealthcheckUnhealthy


@dataclass
class Healthcheck(BaseYamlObject):
    active: HealthcheckActive


@dataclass
class Target(BaseYamlObject):
    target: str


@dataclass
class Upstream(BaseYamlObject):
    name: str
    targets: List[Target]
    healthchecks: Healthcheck



from typing import Optional
from dataclasses import dataclass
from kong_config_builder.base import BaseYamlObject


@dataclass
class KeyauthCredential(BaseYamlObject):
    consumer: str
    key: str


@dataclass
class Plugin(BaseYamlObject):
    name: str
    enabled: bool = True
    service: Optional[str] = None
    route: Optional[str] = None
    config: Optional[dict] = None



from setuptools import setup, find_packages

libs = ["aws"]
extras = {"all": []}

with open("requirements.txt") as reqs:
    requirements = reqs.read().split("\n")

for lib in libs:
    with open(f"requirements_{lib}.txt") as reqs:
        extras[lib] = reqs.read().split("\n")
        extras["all"] = extras["all"] + extras[lib]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="kong_config_builder",
    version="0.0.4",
    description="Kong declarative configuration builder",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Olx",
    license='MIT',
    include_package_data=True,
    url='https://github.com/olxbr/kong-config-builder/',
    download_url='https://github.com/olxbr/kong-config-builder/archive/master.zip',
    install_requires=requirements,
    extra_require=extras,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks"
    ],
    packages=find_packages()
)



from yaml import dump
from typing import List
from dataclasses import dataclass
from kong_config_builder.base import BaseYamlObject
from kong_config_builder.utils import no_empty, noop
from kong_config_builder.consumer import Consumer
from kong_config_builder.plugin import Plugin, KeyauthCredential
from kong_config_builder.service import Service, Route
from kong_config_builder.upstream import (
    Upstream, Target, Healthcheck, HealthcheckActive,
    HealthcheckHealthy, HealthcheckUnhealthy)


@dataclass
class Kong(BaseYamlObject):
    services: List[Service]
    upstreams: List[Upstream]
    plugins: List[Plugin]
    consumers: List[Consumer]
    keyauth_credentials: List[KeyauthCredential]
    _format_version: str = "1.1"

    def save(self, filename="kong.yml"):
        with open(filename, "w") as f:
            dump(self, f)


Kong.yaml_dumper.add_representer(Kong, no_empty)
Kong.yaml_dumper.process_tag = noop

__all__ = [
    "Consumer",
    "Plugin",
    "KeyauthCredential",
    "Service",
    "Route",
    "Upstream",
    "Target",
    "Healthcheck",
    "HealthcheckActive",
    "HealthcheckHealthy",
    "HealthcheckUnhealthy",
    "Kong"
]



import string
import secrets


class PasswordManager:
    @staticmethod
    def generate(size: int) -> str:
        alphabet = string.ascii_letters + string.digits
        return "".join(secrets.choice(alphabet) for i in range(int(size)))
